/*****************************************************************************

This source code is made available to you under the terms of the Open
Development License.  This Package should be accompanied by a file that
contains the associated licensing terms.  If you are unable to find this file,
please refer to its location on the Internet at www.rocklyte.com/license.html.

******************************************************************************

This program is designed to be run from the Athene environment.  It
creates a graphical area with the words "Hello World" printed inside of it.

Note that in this instance we are purposefully using the longest way possible
of writing a Hello World program (fur tutorial's sake).  Realistically, a
simple program like this would normally be written using the Dynamic Markup
Language.

*****************************************************************************/

#include <pandora/system/all.h>
#include <pandora/graphics/all.h>

STRING ProgName      = "Hello World";
STRING ProgAuthor    = "Rocklyte Systems";
STRING ProgDate      = "December 2003";
STRING ProgCopyright = "Copyright Rocklyte Systems (c) 2001-2003.  All rights reserved.";
LONG   ProgDebug = 0;
FLOAT  ProgKernelVersion = 3.3;

extern struct KernelBase *KernelBase;

ERROR msghandler(struct Message *);

OBJECTID glWindowID;

ERROR program(void)
{
   struct Render *render;
   OBJECTPTR window, text;
   OBJECTID render_id;
   LONG topmargin, bottommargin, rightmargin, leftmargin;

   /*** Open a window ***/

   if (CreateObject(ID_WINDOW, NULL, &window, &glWindowID,
      FID_Title|TSTRING,      ProgName,
      FID_InsideBorder|TLONG, TRUE,
      FID_InsideWidth|TLONG,  150,
      FID_InsideHeight|TLONG, 50,
      FID_Icon|TSTRING,       "nature/flower",
      TAGEND) IS ERR_Okay) {

      /*** Get the window margins ***/

      GetFields(window, FID_LeftMargin|TLONG,   &leftmargin,
                        FID_TopMargin|TLONG,    &topmargin,
                        FID_BottomMargin|TLONG, &bottommargin,
                        FID_RightMargin|TLONG,  &rightmargin,
                        TAGEND);

      ReleaseObject(window);

      /*** Create a render area for drawing graphics ***/

      if (CreateObject(ID_RENDER, NULL, &render, &render_id,
         FID_Owner|TLONG,    glWindowID,
         FID_XCoord|TLONG,   leftmargin,
         FID_YCoord|TLONG,   topmargin,
         FID_XOffset|TLONG,  rightmargin,
         FID_YOffset|TLONG,  bottommargin,
         FID_Colour|TSTRING, "#ffffff",
         TAGEND) IS ERR_Okay) {

         /*** Create the Hello World text ***/

         CreateObject(ID_TEXT, NULL, &text, NULL,
            FID_Owner|TLONG,    render_id,
            FID_String|TSTRING, "Hello World",
            FID_Align|TLONG,    ALIGN_CENTER,
            FID_Face|TSTRING,   "Helvete:16",
            FID_Colour|TSTRING, "#000000",
            TAGEND);

         /*** Show the render object inside the window ***/

         Action(AC_Show, render, NULL);
         ReleaseObject(render);

         /*** Show our window ***/

         ActionMsg(AC_Show, glWindowID, NULL);

         /*** Set our message handler and put the task to sleep ***/

         SetResource(RES_MESSAGEHANDLER, (LARGE)&msghandler);

         ProcessMessages(NULL, MF_WAIT);
      }
   }

   return(ERR_Okay);
}

/*****************************************************************************
** Function: msghandler()
**
** This function is called by the system whenever it discovers new
** messages that need to be processed manually.
*/

ERROR msghandler(struct Message *Message)
{
   /* Return a terminate code if MSGID_QUIT is detected (the hello world
   ** window will send us such a message when the user closes it).
   **
   ** By returning the termination code, the ProcessMessages() function will
   ** break out of its sleep-state and our program will leave the system.
   */

   if (Message->Type IS MSGID_QUIT) return(ERR_Terminate);

   return(ERR_Failed); /* Return failure if message unrecognised */
}
