
static void PositionWindow(OBJECTPTR Window)
{
   struct RenderInfo info;
   OBJECTID parent_id;
   OBJECTPTR playerwin;
   LONG x, y, win_width, win_height;
   LONG player_x, player_y, player_width, player_height;

   x = 20;
   y = 20;

   if (AccessObject(glWindowID, 2000, &playerwin) IS ERR_Okay) {
      GetFields(playerwin, FID_XCoord|TLONG, &player_x,
                           FID_YCoord|TLONG, &player_y,
                           FID_Width|TLONG,  &player_width,
                           FID_Height|TLONG, &player_height,
                           FID_Parent|TLONG, &parent_id,
                           TAGEND);

      ReleaseObject(playerwin);

      if (GetRenderInfo(parent_id, &info, sizeof(info)) IS ERR_Okay) {

         x = player_x;

         if (player_y < info.Height / 2) {
            /*** Try to place the playlist just below the player window ***/

            y = player_y + player_height + 5;

            if (y + 40 > info.Height) {
               x = player_x + 20;
               y = player_y + 20;
            }
         }
         else {
            /*** Place the playlist above the player window ***/
            GetField(Window, FID_Height, FT_LONG, &win_height);
            y = player_y - win_height - 5;
         }

         GetField(Window, FID_Width, FT_LONG, &win_width);

         if (x + win_width > info.Width) {
            x = player_x + player_width - win_width;
         }
      }
   }

   ActionTags(AC_MoveToPoint, Window, (DOUBLE)x, (DOUBLE)y, 0.0, MTF_X|MTF_Y);
}

/*****************************************************************************
** Internal: PlaylistWindow()
*/

static void PlaylistWindow(void)
{
   static OBJECTID glPlaylistID = NULL;
   OBJECTPTR window, script, object;
   OBJECTID render_id;
   LONG left, top, bottom, right, width, height;
   WORD i;

   /*** Use some intelligence in determining the position of the playlist window ***/

   if ((glPlaylistID) AND (CheckObjectExists(glPlaylistID, NULL) IS ERR_Okay)) {
      if (AccessObject(glPlaylistID, 3000, &window) IS ERR_Okay) {
         PositionWindow(window);
         Action(AC_MoveToFront, window, NULL);
         Action(AC_Focus, window, NULL);
         ReleaseObject(window);
      }
      return;
   }

   if (CreateObject(ID_WINDOW, NULL, &window, &glPlaylistID,
      FID_Title|TSTRING,      "Play List",
      FID_InsideWidth|TLONG,  300,
      FID_InsideHeight|TLONG, 150,
      FID_MinWidth|TLONG,     160,
      FID_MinHeight|TLONG,    120,
      FID_Icon|TSTRING,       "office/documents",
      FID_Quit|TLONG,         FALSE,
      TAGEND) IS ERR_Okay) {

      PositionWindow(window);

      GetFields(window, FID_LeftMargin|TLONG,   &left,
                        FID_TopMargin|TLONG,    &top,
                        FID_BottomMargin|TLONG, &bottom,
                        FID_RightMargin|TLONG,  &right,
                        TAGEND);

      /*** Create buttons ***/

      if (CreateObject(ID_BUTTON, NULL, &object, NULL,
         FID_Owner|TLONG,    glPlaylistID,
         FID_String|TSTRING, "Add Files",
         FID_YOffset|TLONG,  bottom,
         FID_XCoord|TLONG,   left,
         TAGEND) IS ERR_Okay) {

         glPLAdd = object->UniqueID;

         SubscribeAction(object, AC_Activate, glTask);

         Action(AC_Show, object, NULL);

         GetFields(object, FID_Width|TLONG,  &width,
                           FID_Height|TLONG, &height,
                           TAGEND);
      }
      else {
         Action(AC_Free, window, NULL);
         ReleaseObject(window);
         return;
      }

      if (CreateObject(ID_BUTTON, NULL, &object, NULL,
         FID_Owner|TLONG,    glPlaylistID,
         FID_String|TSTRING, "Delete",
         FID_YOffset|TLONG,  bottom,
         FID_XCoord|TLONG,   left + width + 6,
         TAGEND) IS ERR_Okay) {

         glPLDelete = object->UniqueID;

         SubscribeAction(object, AC_Activate, glTask);

         Action(AC_Show, object, NULL);
      }
      else {
         Action(AC_Free, window, NULL);
         ReleaseObject(window);
         return;
      }

      if (CreateObject(ID_RENDER, NULL, NULL, &render_id,
         FID_Owner|TLONG,    glPlaylistID,
         FID_XCoord|TLONG,   left,
         FID_YCoord|TLONG,   top,
         FID_XOffset|TLONG,  right,
         FID_YOffset|TLONG,  bottom + height + 6,
         FID_Colour|TSTRING, "#ffffe0",
         TAGEND) IS ERR_Okay) {

         if (CreateObject(ID_SCRIPT, NF_CHILD, &script, NULL,
            FID_Target|TLONG,     render_id,
            FID_Location|TSTRING, "templates:insideborder",
            FID_Input|TUNLISTED,  "1",
            FID_Fill|TUNLISTED,   "1",
            TAGEND) IS ERR_Okay) {
            Action(AC_Activate, script, NULL);
            Action(AC_Free, script, NULL);
         }

         CreateObject(ID_SCROLLBAR, NULL, NULL, NULL,
            FID_Owner|TLONG, render_id,
            FID_Direction|TSTRING, "vertical",
            FID_YCoord|TTRANS,     "[gl_interface.inputborder_y]",
            FID_XOffset|TTRANS,    "[gl_interface.inputborder_xo]",
            FID_YOffset|TTRANS,    "[gl_interface.inputborder_yo]",
            TAGEND);

         CreateObject(ID_SCROLLBAR, NULL, NULL, NULL,
            FID_Owner|TLONG, render_id,
            FID_Direction|TSTRING, "horizontal",
            FID_XCoord|TTRANS,     "[gl_interface.inputborder_x]",
            FID_XOffset|TTRANS,    "[gl_interface.inputborder_xo]",
            FID_YOffset|TTRANS,    "[gl_interface.inputborder_yo]",
            TAGEND);

         if (CreateObject(ID_VIEW, NULL, &glPlaylistView, NULL,
            FID_Owner|TLONG,                 render_id,
            FID_BorderOffset|TTRANS|TSTRING, "[gl_interface.inputborder]",
            FID_Flags|TLONG,                 VWF_MULTISELECT,
            FID_Columns|TSTRING,             "track(text:Track, len:40); default(text:Title, len:200); length(text:Length, len:50, type:numeric)",
            TAGEND) IS ERR_Okay) {

            glPlaylistViewID = glPlaylistView->UniqueID;

            SubscribeField(glPlaylistView, FID_SelectedTag, glTask);
            SubscribeAction(glPlaylistView, AC_Activate, glTask);

            for (i=0; i < glPlayIndex; i++) {
               AddViewXML(i);
            }
         }

         ActionMsg(AC_Show, render_id, NULL);
      }

      Action(AC_Show, window, NULL);

      ReleaseObject(window);
   }
}

/*****************************************************************************
** Internal: AddPlayFile()
*/

static void AddPlayFile(STRING File)
{
   OBJECTPTR object;
   CLASSID classid;
   LONG length, bytespersecond, freq, bits;

   DPrintF("AddPlayFile()","%s", File);

   if (glPlayIndex < ARRAYSIZE(glPlaylist)) {
      if (IdentifyFile(File, &classid, NULL) IS ERR_Okay) {
         if ((classid != ID_MUSIC) AND (classid != ID_SOUND)) return;

         glPlaylist[glPlayIndex].File = StrClone(File);
         glPlaylist[glPlayIndex].Length = 0;

         /* Determine the length of the file, measured in seconds, and
         ** get information such as the title, frequency etc
         */

         if (classid IS ID_MUSIC) {
            if (CreateObject(ID_MUSIC, NULL, &object, NULL,
               FID_Location|TSTRING, File,
               TAGEND) IS ERR_Okay) {
               GetField(object, FID_SongLength, FT_LONG, &glPlaylist[glPlayIndex].Length);
               glPlaylist[glPlayIndex].Title = GetTitle(object, File);

               acFree(object);
            }
         }
         else if (classid IS ID_SOUND) {
            if (CreateObject(ID_SOUND, NULL, &object, NULL,
               FID_Location|TSTRING, File,
               TAGEND) IS ERR_Okay) {
               GetFields(object, FID_Length|TLONG,         &length,
                                 FID_BytesPerSecond|TLONG, &bytespersecond,
                                 FID_Frequency|TLONG,      &freq,
                                 FID_BitsPerSample|TLONG,  &bits,
                                 TAGEND);

               if ((length) AND (bytespersecond)) {
                  glPlaylist[glPlayIndex].Length = length / bytespersecond;
                  if (glPlaylist[glPlayIndex].Length < 1) glPlaylist[glPlayIndex].Length = 1;
               }

               glPlaylist[glPlayIndex].Title = GetTitle(object, File);

               acFree(object);
            }
         }

         if (!glPlaylist[glPlayIndex].Title) glPlaylist[glPlayIndex].Title = GetTitle(NULL, File);

         /*** If the playlist window is open, add the file to it ***/

         if ((glPlaylistViewID) AND (CheckObjectIDExists(glPlaylistViewID) IS ERR_Okay)) {
            AddViewXML(glPlayIndex);
         }

         glPlayIndex++;
      }
   }
   else print("The playlist is full.");

   PrintPlaylist();
}

/*****************************************************************************
** Internal: RemovePlayFile()
*/

static void RemovePlayFile(LONG Index)
{
   if (glPlaylist[Index].File) {
      FreeMemory(glPlaylist[Index].File);
      glPlaylist[Index].File = NULL;

      if (glPlaylist[Index].Title) {
         FreeMemory(glPlaylist[Index].Title);
         glPlaylist[Index].Title = NULL;
      }

      if (Index < glPlayIndex-1) {
         CopyMemory(glPlaylist + Index + 1, glPlaylist + Index, sizeof(glPlaylist[0]) * (glPlayIndex - Index - 1));
      }
      glPlayIndex--;
   }
}

/*****************************************************************************
** Internal: ClearPlayList()
*/

static void ClearPlaylist(void)
{
   WORD i;

   for (i=0; i < glPlayIndex; i++) {
      if (glPlaylist[i].File) FreeMemory(glPlaylist[i].File);
      if (glPlaylist[i].Title) FreeMemory(glPlaylist[i].Title);
      glPlaylist[i].File = NULL;
      glPlaylist[i].Title = NULL;
   }

   glPlayIndex = 0;
   glTrack = 0;

   if ((glPlaylistViewID) AND (CheckObjectIDExists(glPlaylistViewID) IS ERR_Okay)) {
      acClear(glPlaylistView);
   }
}
